<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" 
        xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
        xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xsl:strip-space elements="*"/>
    <xsl:output method="text" /> 

<!-- Copyright Kathleen Dollard 2002 -->
    
<!-- Main template that calls other templates for fragments -->
<xsl:template name="DerivedRow">
   <xsl:variable name="Table" select="@name" />
    &lt;System.Diagnostics.DebuggerStepThrough()>  _
   Public Class <xsl:value-of select="$Table"/>Row
      Inherits DataRow

      Private table<xsl:value-of select="$Table"/> As <xsl:value-of select="@name"/>DataTable

    <xsl:call-template name="RowConstructors" />
    <xsl:apply-templates select="DataColumn" mode="RowColumnProperties">
		<xsl:with-param name="Table" select="$Table"/>
    </xsl:apply-templates>
    <xsl:apply-templates select="//Relation[ChildTable=$Table]" mode="RowGetParentRows" />
    <xsl:apply-templates select="DataColumn[not(@AllowDBNulls='false')]" mode="RowColumnNullStuff">
		<xsl:with-param name="Table" select="$Table"/>
    </xsl:apply-templates>
    <xsl:apply-templates select="//Relation[ParentTable=$Table]" mode="RowGetChildRows" />
   End Class
</xsl:template>

<!-- Fragment Templates -->   
<xsl:template name="RowConstructors">
        Friend Sub New(ByVal rb As DataRowBuilder)
            MyBase.New(rb)
            Me.table<xsl:value-of select="@name"/> = CType(Me.Table,<xsl:value-of select="@name"/>DataTable)
        End Sub
</xsl:template>

<xsl:template match="DataColumn" mode="RowColumnProperties">
		  <xsl:param name="Table"/>
        Public Property <xsl:value-of select="@name"/> As <xsl:value-of select="@VBType"/>
            Get<xsl:choose>
				  <xsl:when test="not(@AllowDBNulls)">
                Try
	                Return CType( _
							Me(Me.table<xsl:value-of select="$Table"/>. _
							<xsl:value-of select="@name"/>Column), _
							<xsl:value-of select="@VBType"/>)
                Catch e As InvalidCastException
                    Throw New StrongTypingException("Cannot get value because it is DBNull.", e)
                End Try	</xsl:when>
				  <xsl:otherwise>
                Return CType(Me(Me.table<xsl:value-of select="$Table"/>.<xsl:value-of select="@name"/>Column),<xsl:value-of select="@VBType"/>)
				  </xsl:otherwise>
				</xsl:choose>
            End Get
            Set
                Me(Me.table<xsl:value-of select="$Table"/>.<xsl:value-of select="@name"/>Column) = value
            End Set
        End Property
</xsl:template>

<xsl:template match="//Relation" mode="RowGetParentRows">
        Public Property <xsl:value-of select="ParentTable"/>Row As <xsl:value-of select="ParentTable"/>Row
            Get
                Return CType(Me.GetParentRow(Me.Table.ParentRelations("<xsl:value-of select="@name"/>")),<xsl:value-of select="ParentTable"/>Row)
            End Get
            Set
                Me.SetParentRow(value, Me.Table.ParentRelations("<xsl:value-of select="@name"/>"))
            End Set
        End Property
</xsl:template>    

<xsl:template match="DataColumn" mode="RowColumnNullStuff">
		  <xsl:param name="Table"/>
        Public Function Is<xsl:value-of select="@name"/>Null() As Boolean
            Return Me.IsNull(Me.table<xsl:value-of select="$Table"/>.<xsl:value-of select="@name"/>Column)
        End Function
        
        Public Sub Set<xsl:value-of select="@name"/>Null()
            Me(Me.table<xsl:value-of select="$Table"/>.<xsl:value-of select="@name"/>Column) = System.Convert.DBNull
        End Sub
</xsl:template>

<xsl:template match="//Relation" mode="RowGetChildRows">
        Public Function Get<xsl:value-of select="ChildTable"/>Rows() As <xsl:value-of select="ChildTable"/>Row()
            Return CType(Me.GetChildRows(Me.Table.ChildRelations("<xsl:value-of select="@name"/>")),<xsl:value-of select="ChildTable"/>Row())
        End Function
</xsl:template>    
    
</xsl:stylesheet>

  