package net.matuschek.jobo.task;

/*********************************************
    Copyright (c) 2001 by Daniel Matuschek
    Contributor: Didier Donsez (2004)
*********************************************/

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import net.matuschek.http.HttpDocToFile;
import net.matuschek.http.HttpToolCallback;
import net.matuschek.jobo.JoBoBase;
import net.matuschek.spider.WebRobot;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Category;
import org.apache.log4j.PropertyConfigurator;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

/**
 * This is the JoBo command line interface.
 * 
 * @author Daniel Matuschek 
 * @version $Id $
 */
public class JoBoTask extends Task {

	protected static Category log = Category.getInstance("");

	//-v]              verbose mode, useful, if something is wrong
	//					 with the XML configuration

	//	-s directory]    directory to store retrieved documents  (default \".\")
	private String dest = null;
	private boolean debug = false;
	//-o]              allow walk to other hosts (default no)
	private boolean walkToOtherHosts = false;
	//-i]              ignore robots.txt
	private boolean ignoreRobotsTxt = true;

	//-r http://...]   set start referer (default \"-\")
	private String referer = null;
	//-d maxdepth]     set maximal search depth (default 0)
	private String maximalDepth = null;
	//-m minsize]      store only files larger then this size in bytes (default 0)
	private String minimalFileSize = null;
	//-a agentName]    set user agent name (default \"JoBo\")
	private String agentName = null;
	//-w seconds]      wait n seconds after retrieving a file to limit load on the remote server (default 60)
	private int waitTime = 0;

	private String configfile = "logging.conf";
	//url               start URL
	private String url = null;

	/**
	 initialize log4j logging subsystem
	**/
	public void initializeLogging() {
		File f = new File(configfile);
		if (f.exists()) {
			// read the logging properties from configuration file
			PropertyConfigurator.configure(configfile);
		} else {
			BasicConfigurator.configure();
		}
	}

	public void execute() throws BuildException {
		String basedir = ".";
		int minSize = 0;

		initializeLogging();

		JoBoBase jobobase;
		try {
			jobobase = JoBoBase.createFromXML();
		} catch (ClassNotFoundException e1) {
			throw new BuildException(e1);
		}
		WebRobot robby = jobobase.getRobot();

		if (referer != null) {
			robby.setStartReferer(referer);
		}

		// maximalDepth
		if (maximalDepth != null) {
			try {
				int maxDepth = Integer.parseInt(maximalDepth);
				robby.setMaxDepth(maxDepth);
			} catch (NumberFormatException e) {
				throw new BuildException(
					"Wrong number for maxDepth: " + maximalDepth);
			}
		}

		robby.setWalkToOtherHosts(walkToOtherHosts);

		// store directory
		if (dest != null) {
			basedir = dest;
		} else {
			throw new BuildException("Attribute dest is missing");
		}

		// minimal file size
		if (minimalFileSize != null) {
			try {
				minSize = Integer.parseInt(minimalFileSize);
			} catch (NumberFormatException e) {
				throw new BuildException(e);
			}
		}

		// agent name
		if (agentName != null) {
			robby.setAgentName(agentName);
		}

		// ignore robots.txt
		robby.setIgnoreRobotsTxt(ignoreRobotsTxt);

		// wait time
		robby.setSleepTime(waitTime * 1000);

		// url
		if (url == null) {
			throw new BuildException("Attribute url is missing");
		}

		URL u;
		try {
			u = new URL(url);
		} catch (MalformedURLException e) {
			throw new BuildException(e);
		}

		HttpDocToFile docStore = new HttpDocToFile(basedir);
		docStore.setMinFileSize(minSize);

		HttpToolCallback statusInfo = new LogHttpToolCallBack(this);

		robby.setStartURL(u);
		robby.setDocManager(docStore);
		robby.setHttpToolCallback(statusInfo);

		robby.run();

	}

	/**
	 * @param string
	 */
	public void setAgentName(String string) {
		agentName = string;
	}

	/**
	 * @param string
	 */
	public void setConfigfile(String string) {
		configfile = string;
	}

	/**
	 * @param b
	 */
	public void setDebug(boolean b) {
		debug = b;
	}

	/**
	 * @param b
	 */
	public void setIgnoreRobotsTxt(boolean b) {
		ignoreRobotsTxt = b;
	}

	/**
	 * @param string
	 */
	public void setMaximalDepth(String string) {
		maximalDepth = string;
	}

	/**
	 * @param string
	 */
	public void setMinimalFileSize(String string) {
		minimalFileSize = string;
	}

	/**
	 * @param string
	 */
	public void setReferer(String string) {
		referer = string;
	}

	/**
	 * @param i
	 */
	public void setWaitTime(int i) {
		waitTime = i;
	}

	/**
	 * @param b
	 */
	public void setWalkToOtherHosts(boolean b) {
		walkToOtherHosts = b;
	}

	/**
	 * @param string
	 */
	public void setUrl(String string) {
		url = string;
	}

	/**
	 * @param string
	 */
	public void setDest(String string) {
		dest = string;
	}

}
