/*
 * Copyright  2004 Didier Donsez
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 * TODO: add attribute scriptfile
 * TODO: add attribute classpath for the jdbc driver
 * TODO: add mapper to map token names  for instance <mapper type="glob" from="*" to="__*__"/>
 *
 * TODO: test and document following attributes for sql based field replacement

 */
package fr.imag.adele.ant.templatetask;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Replace;
import org.apache.tools.ant.taskdefs.Replace.Replacefilter;
//import org.apache.tools.ant.taskdefs.optional.Script;

/**
 *  The task generates several files by replacing fields into a template file
 *  Field values are stored in a JDBC datasource or in a CSV (Comma Separated Values) formatted file.
 *
 *@author     Didier Donsez
 *@version    March 10, 2004
 */
public class TemplateTask extends Task {

	protected String CSV_SEPARATOR = ";";
	protected String DEFAULTNULLVALUE = "";
	
	protected File template; // template file
	protected File data; // data file
	protected String targetname; // template filename of the generated files 
	protected File dir; // directory in which files are generated
	protected String ranktoken; // the rank token string
	protected int rankstart = 1; // the start number of the rank value
	protected String scriptText;
	// the text of the script to generate and execute
	protected boolean forceOverwrite = false;
	protected String language; // the language of the script

	protected String driver; // Class name of the jdbc driver Yes 
	protected String dburl; //  Database connection url Yes 
	protected String userid; //  Database user name Yes 
	protected String password; //  Database password Yes 
	protected String statement; //  the sql select statement

	/**
	 * call by ant
	 *
	 *@exception  BuildException  Description of the Exception
	 */
	public void execute() throws BuildException {

		if (template == null) {
			new BuildException("Attribute template is required");
		}
		if (data == null) {
			new BuildException("Attribute data is required");
		}
		if (dir == null) {
			new BuildException("Attribute dir is required");
		}
		if (targetname == null) {
			new BuildException("Attribute targetnane is required");
		}

		if (data != null && dburl !=null) {
			new BuildException("Data source should be either a CSV file or a JDBC datasource");	
		}


		if (data != null) {
			Project project = getProject();
			BufferedReader templatebr;
			try {
				templatebr = new BufferedReader(new FileReader(template));
			} catch (FileNotFoundException e) {
				throw new BuildException("File " + template + " not found");
			}
			BufferedReader databr;
			try {
				databr = new BufferedReader(new FileReader(data));
			} catch (FileNotFoundException e) {
				throw new BuildException("File " + data + " not found");
			}

			String line = null;
			String headerline = null;
			int rank = rankstart;
			try {
				while ((line = databr.readLine()) != null) {

					if (headerline == null) {
						headerline = line;
					} else {

						Copy copy = new Copy();
						copy.setTaskName(getTaskName());
						copy.setProject(getProject());
						copy.setOverwrite(forceOverwrite);
						copy.setFile(template);

						Replace replace = new Replace();
						replace.setTaskName(getTaskName());
						replace.setProject(getProject());

						//					Script script=null;
						if (scriptText != null) {
							//						script=new Script();
							//						script.setTaskName(getTaskName());
							//						script.setProject(getProject());
							//						script.setLanguage(language);
						}

						StringTokenizer headerst =
							new StringTokenizer(headerline, CSV_SEPARATOR);
						StringTokenizer linest =
							new StringTokenizer(line, CSV_SEPARATOR);
						StringBuffer replacetargetnamesb =
							new StringBuffer(targetname);
						StringBuffer scriptsb = null;
						if (scriptText != null)
							scriptsb = new StringBuffer(scriptText);

						while (headerst.hasMoreTokens()) {
							String headerfield = headerst.nextToken();
							String linefield = null;
							if (linest.hasMoreTokens()) {
								linefield = linest.nextToken();
							} else {
								linefield = "";
							}
							stringBufferReplace(
								replacetargetnamesb,
								headerfield,
								linefield);
							if (scriptText != null)
								stringBufferReplace(
									scriptsb,
									headerfield,
									linefield);

							Replacefilter filter =
								replace.createReplacefilter();
							filter.setToken(headerfield);
							filter.setValue(linefield);
						}

						if (ranktoken != null) {
							String rankSt = (new Integer(rank)).toString();
							stringBufferReplace(
								replacetargetnamesb,
								ranktoken,
								rankSt);
							if (scriptText != null)
								stringBufferReplace(
									scriptsb,
									ranktoken,
									rankSt);

							Replacefilter filter =
								replace.createReplacefilter();
							filter.setToken(ranktoken);
							filter.setValue(rankSt);
							rank++;
						}

						String replacetargetname =
							replacetargetnamesb.toString();
						File replacetarget = new File(dir, replacetargetname);
						copy.setTofile(replacetarget);
						replace.setFile(replacetarget);

						log(
							"Generated file "
								+ replacetarget.getAbsolutePath()
								+ " for "
								+ line);
						copy.execute();
						replace.execute();
						if (scriptText != null) {
							//						script.addText(scriptsb.toString());
							//						script.execute();
						}
					}
				}
			} catch (IOException e) {
				throw new BuildException(
					"IO Error occured during reading file" + template);
			}
		} else if (dburl != null) {

			Connection cnx = null;
			ResultSet rs = null;
			
			try {
				try {
					//this.getClass().getClassLoader().loadClass(driver);
					Class.forName(driver);
				} catch (ClassNotFoundException e1) {
					throw new BuildException(
						"Cannot find driver class:" + driver);
				}
				cnx = DriverManager.getConnection(dburl, userid, password);
				PreparedStatement stmt = cnx.prepareStatement(statement);
				if (stmt.execute()) {
					rs = stmt.getResultSet();
					ResultSetMetaData rsmd = rs.getMetaData();
					int numcols = rsmd.getColumnCount();

					int rank = rankstart;
					while (rs.next()) {

						Copy copy = new Copy();
						copy.setTaskName(getTaskName());
						copy.setProject(getProject());
						copy.setOverwrite(forceOverwrite);
						copy.setFile(template);

						Replace replace = new Replace();
						replace.setTaskName(getTaskName());
						replace.setProject(getProject());

						StringBuffer replacetargetnamesb =
							new StringBuffer(targetname);

						for (int i = 1; i <= numcols; i++) {
							String headerfield = rsmd.getColumnLabel(i);
							Object obj = rs.getObject(i);
							String linefield;
							if (obj != null) {
								linefield = obj.toString();
							} else {
								linefield = DEFAULTNULLVALUE;
							}
							stringBufferReplace(
								replacetargetnamesb,
								headerfield,
								linefield);

							Replacefilter filter =
								replace.createReplacefilter();
							filter.setToken(headerfield);
							filter.setValue(linefield);
						}

						if (ranktoken != null) {
							String rankSt = (new Integer(rank)).toString();
							stringBufferReplace(
								replacetargetnamesb,
								ranktoken,
								rankSt);

							Replacefilter filter =
								replace.createReplacefilter();
							filter.setToken(ranktoken);
							filter.setValue(rankSt);
						}
						rank++;

						String replacetargetname =
							replacetargetnamesb.toString();
						File replacetarget = new File(dir, replacetargetname);
						copy.setTofile(replacetarget);
						replace.setFile(replacetarget);

						log(
							"Generated file "
								+ replacetarget.getAbsolutePath()
								+ " for tuple "
								+ (rank-1) );
						copy.execute();
						replace.execute();
						
					}
					rs.close();
				}
				cnx.close();
			} catch (SQLException e) {
				try{
					if(rs!=null) rs.close();
					if(cnx!=null) cnx.close();
				} catch(SQLException se){
					// do nothing !
				}
				throw new BuildException(
					"Error occured during querying jdbc source: "+dburl);
			}
		}
	}

	/**
	 *  Sets the template attribute of the TemplateTask object
	 *
	 *@param  template  The new template value
	 */
	public void setTemplate(File template) {
		this.template = template;
	}

	/**
	 *  Sets the data attribute of the TemplateTask object
	 *
	 *@param  data  The new data value
	 */
	public void setData(File data) {
		this.data = data;
	}

	/**
	 *  Sets the targetnane attribute of the TemplateTask object
	 *
	 *@param  targetnane  The new targetnane value
	 */
	public void setTargetname(String targetname) {
		this.targetname = targetname;
	}

	/**
	 *  Sets the dir attribute of the TemplateTask object
	 *
	 *@param  dir  The new dir value
	 */
	public void setDir(File dir) {
		this.dir = dir;
	}

	/**
	 *  Sets the ranktoken attribute of the TemplateTask object
	 *
	 *@param  ranktoken  The new ranktoken value
	 */
	public void setRanktoken(String ranktoken) {
		this.ranktoken = ranktoken;
	}

	/**
	 *  Sets the ranktoken attribute of the TemplateTask object
	 *
	 *@param  ranktoken  The new ranktoken value
	 */
	public void setRankstart(int rankstart) {
		this.rankstart = rankstart;
	}

	/**
	 * Overwrite any existing destination file(s).
	 * @param overwrite if true force overwriting of destination file(s)
	 *                  even if the destination file(s) are younger than
	 *                  the corresponding source file. Default is false.
	 */
	public void setOverwrite(boolean overwrite) {
		this.forceOverwrite = overwrite;
	}

	/**
	 * Defines the language (required).
	 *
	 * @param language the scripting language name for the script.
	 */
	//	public void setLanguage(String language) {
	//		this.language = language;
	//	}

	/**
	 *  Adds a feature to the Script attribute of the TemplateTask object
	 *
	 *@param  text  The feature to be added to the Script attribute
	 */
	//	public void addText(String scriptText) {
	//		this.scriptText = scriptText;
	//	}

	/**
	 * @param dburl the database connection url
	 */
	public void setDburl(String dburl) {
		this.dburl = dburl;
	}

	/**
	 * @param driver the class name of the jdbc driver Yes 
	 */
	public void setDriver(String driver) {
		this.driver = driver;
	}

	/**
	 * @param password the database password
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * @param statement the sql select statement
	 */
	public void setStatement(String statement) {
		this.statement = statement;
	}

	/**
	 * @param userid the database user name
	 */
	public void setUserid(String userid) {
		this.userid = userid;
	}

	/**
	 * Replace occurrences of str1 in string str with str2
	 */
	private int stringBufferReplace(
		StringBuffer sb,
		String str1,
		String str2) {
		int replaceCount = 0;
		int start = 0;
		int found;
		while ((found = sb.indexOf(str1)) >= 0) {
			sb.replace(found, found + str1.length(), str2);
			++replaceCount;
		}

		return replaceCount;
	}

}
